'use strict';

const _ = require('lodash');
const GooglePubsub = require('@google-cloud/pubsub');
const PubsubSubscription = require('./pubsub-subscription');

class PubsubTopic {

    constructor(id, options) {
        this.options = _.defaults(options || {}, {
            credentials: null,
            autoAck: false,
        });

        this.subscriptions = {};

        this.id = id;
        this.topic = null;
        this.link = null;
    }

    getLink() {
        if(!this.link) {
            this.link = GooglePubsub(this.options.credentials);
        }

        return this.link;
    }

    getTopic() {
        if(!this.topic) {
            this.topic = this.create(this.id);
        }

        return this.topic;
    }

    create(id) {
        let topic = this.getLink().topic(id);

        return topic
            .get({
                autoCreate: true,
            })
            .then(data => data[0]);
    }

    /**
     * @param id
     * @param options
     * @returns {PubsubSubscription}
     */
    subscribe(id, options) {
        if(!(id in this.subscriptions)) {
            this.subscriptions[id] = new PubsubSubscription(id, this, _.defaults(options, this.options));
        }

        return this.subscriptions[id];
    }

    publish(msg, options) {
        options = _.defaults(options || {}, {
            raw: null,
            timeout: null,
            delay: null,
        });

        if(options.delay) {
            return this.publishDelayed(msg, options);
        }

        return this
            .getTopic()
            .then(topic => {
                return topic
                    .publish(msg, options)
                    .then(data => {
                        if(!_.isArray(msg)) {
                            return data[0][0];
                        }

                        return data[0];
                    });
            });
    }

    publishDelayed(msg, options) {
        options = _.defaults(options, {
            delay: null,
        });

        // 1. save to datastore
    }

    destroy() {
        if(!this.topic) {
            return Promise.resolve();
        }

        return this
            .getTopic()
            .then(topic => {
                return topic
                    .delete()
                    .then(data => true);
            })
            .then(() => {
                this.topic = null;
            });
    }
}

module.exports = PubsubTopic;